local Cryolysis2 = Cryolysis2
local L = AceLibrary("AceLocale-2.2"):new("Cryolysis2")
local BS = AceLibrary("Babble-Spell-2.2")

------------------------------------------------------------------------------------------------------
-- Function to find a specific spell type
------------------------------------------------------------------------------------------------------
function Cryolysis2:FindSpellType(spellName)
	for i = 1, MAX_SKILLLINE_TABS do
		local name, texture, offset, numSpells = GetSpellTabInfo(i);
		   
		if not name then
			break;
		end
		   
		for s = offset + 1, offset + numSpells do
			local spell, rank = GetSpellName(s, BOOKTYPE_SPELL);
			if spell == spellName then
				return i
			end
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Function to find a specific spell id
------------------------------------------------------------------------------------------------------
function Cryolysis2:FindSpellID(findname)

	local i = 1
	if string.len(findname) > 0 then
		while true do
			local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
			if not spellName then
				do break end
			end
			
			if spellName == findname then
				return i
			end
			i = i + 1
		end
	end
	return 0

end

------------------------------------------------------------------------------------------------------
-- Function used to detect what spells the mage has.
-- For food and water it also detects the highest rank the mage has.
-- This is so we can add in casting of a certain rank of the spell according to the target's level.
-- This function should be called once initially while loading in, and then anytime the mage learns a new spell.
-- THESE STRINGS NEED TO BE LOCALIZED!
------------------------------------------------------------------------------------------------------
function Cryolysis2:DetectSpellsAndRanks()
	local i = 1
	local spellRankText, spellRankNumber
	while true do
		local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
		if not spellName then
			do break end
		end
		
		-- These ones are straight-forward, if we find the spell, set the corresponding value to true
		if string.find(spellName, BS["Arcane Intellect"]) then
			Cryolysis2.Private.HasArcaneIntellect = true
			Cryolysis2.Private.HasSpellMenuBuff[2] = true
		end
		if string.find(spellName, BS["Arcane Brilliance"]) then
			Cryolysis2.Private.HasArcaneBrilliance = true
			Cryolysis2.Private.HasSpellMenuBuff[2] = true
		end
		if string.find(spellName, BS["Frost Armor"]) then
			Cryolysis2.Private.HasFrostArmor = true
			Cryolysis2.Private.HasSpellMenuBuff[1] = true
		end
		if string.find(spellName, BS["Ice Armor"]) then
			Cryolysis2.Private.HasIceArmor = true
			Cryolysis2.Private.HasSpellMenuBuff[1] = true
		end
		if string.find(spellName, BS["Molten Armor"]) then
			Cryolysis2.Private.HasMoltenArmor = true
			Cryolysis2.Private.HasSpellMenuBuff[1] = true
		end
		if string.find(spellName, BS["Mage Armor"]) then
			Cryolysis2.Private.HasMageArmor = true
			Cryolysis2.Private.HasSpellMenuBuff[1] = true
		end
		if string.find(spellName, BS["Remove Lesser Curse"]) then
			Cryolysis2.Private.HasRemoveCurse = true
			Cryolysis2.Private.HasSpellMenuBuff[6] = true
		end
		if string.find(spellName, BS["Slow Fall"]) then
			Cryolysis2.Private.HasSlowFall = true
			Cryolysis2.Private.HasSpellMenuBuff[7] = true
		end
		if string.find(spellName, BS["Evocation"]) then
			Cryolysis2.Private.HasEvocation = true
		end
		if string.find(spellName, BS["Mana Shield"]) then
			Cryolysis2.Private.HasManaShield = true
			Cryolysis2.Private.HasSpellMenuBuff[4] = true
		end
		if string.find(spellName, BS["Ice Barrier"]) then
			Cryolysis2.Private.HasIceBarrier = true
			Cryolysis2.Private.HasSpellMenuBuff[4] = true
		end
		if string.find(spellName, BS["Dampen Magic"]) then
			Cryolysis2.Private.HasDampenMagic = true
			Cryolysis2.Private.HasSpellMenuBuff[3] = true
		end
		if string.find(spellName, BS["Amplify Magic"]) then
			Cryolysis2.Private.HasAmplifyMagic = true
			Cryolysis2.Private.HasSpellMenuBuff[3] = true
		end
		if string.find(spellName, BS["Fire Ward"]) then
			Cryolysis2.Private.HasFireWard = true
			Cryolysis2.Private.HasSpellMenuBuff[5] = true
		end
		if string.find(spellName, BS["Frost Ward"]) then
			Cryolysis2.Private.HasFrostWard = true
			Cryolysis2.Private.HasSpellMenuBuff[5] = true
		end
		
		-- Teleports and Portals
		if string.find(spellName, BS["Teleport: Ironforge"]) or string.find(spellName, BS["Teleport: Orgrimmar"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuTeleport[1] = BS["Teleport: Ironforge"]
			else
				Cryolysis2.Private.HasPortMenuTeleport[1] = BS["Teleport: Orgrimmar"]
			end
			Cryolysis2.Private.HasATeleport = true
		end
		if string.find(spellName, BS["Teleport: Stormwind"]) or string.find(spellName, BS["Teleport: Undercity"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuTeleport[2] = BS["Teleport: Stormwind"]
			else
				Cryolysis2.Private.HasPortMenuTeleport[2] = BS["Teleport: Undercity"]
			end
			Cryolysis2.Private.HasATeleport = true
		end
		if string.find(spellName, BS["Teleport: Darnassus"]) or string.find(spellName, BS["Teleport: Thunder Bluff"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuTeleport[3] = BS["Teleport: Darnassus"]
			else
				Cryolysis2.Private.HasPortMenuTeleport[3] = BS["Teleport: Thunder Bluff"]
			end
			Cryolysis2.Private.HasATeleport = true
		end
		if string.find(spellName, BS["Teleport: Exodar"]) or string.find (spellName, BS["Teleport: Silvermoon"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuTeleport[4] = BS["Teleport: Exodar"]
			else
				Cryolysis2.Private.HasPortMenuTeleport[4] = BS["Teleport: Silvermoon"]
			end
			Cryolysis2.Private.HasATeleport = true
		end
		if string.find(spellName, BS["Teleport: Shattrath"]) then
			Cryolysis2.Private.HasPortMenuTeleport[5] = BS["Teleport: Shattrath"]
			Cryolysis2.Private.HasATeleport = true
		end
		if string.find(spellName, BS["Portal: Ironforge"]) or string.find(spellName, BS["Portal: Orgrimmar"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuPort[1] = BS["Portal: Ironforge"]
			else
				Cryolysis2.Private.HasPortMenuPort[1] = BS["Portal: Orgrimmar"]
			end
			Cryolysis2.Private.HasAPort = true
		end
		if string.find(spellName, BS["Portal: Stormwind"]) or string.find(spellName, BS["Portal: Undercity"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuPort[2] = BS["Portal: Stormwind"]
			else
				Cryolysis2.Private.HasPortMenuPort[2] = BS["Portal: Undercity"]
			end
			Cryolysis2.Private.HasAPort = true
		end
		if string.find(spellName, BS["Portal: Darnassus"]) or string.find(spellName, BS["Portal: Thunder Bluff"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuPort[3] = BS["Portal: Darnassus"]
			else
				Cryolysis2.Private.HasPortMenuPort[3] = BS["Portal: Thunder Bluff"]
			end
			Cryolysis2.Private.HasAPort = true
		end
		if string.find(spellName, BS["Portal: Exodar"]) or string.find (spellName, BS["Portal: Silvermoon"]) then
			if Cryolysis2.Private.englishFaction == "Alliance" then
				Cryolysis2.Private.HasPortMenuPort[4] = BS["Portal: Exodar"]
			else
				Cryolysis2.Private.HasPortMenuPort[4] = BS["Portal: Silvermoon"]
			end
			Cryolysis2.Private.HasAPort = true
		end
		if string.find(spellName, BS["Portal: Shattrath"]) then
			Cryolysis2.Private.HasPortMenuPort[5] = BS["Portal: Shattrath"]
			Cryolysis2.Private.HasAPort = true
		end
		
		-- These ones are slightly more complex.  If we find the spell, we need to check the rank against the currently known rank.
		-- If the rank we found is higher, then we change the highest know rank to the one we found, otherwise we do nothing.
		-- For the mana stones, we have to statically set the rank numbers to compare, since there's no rank information for them.
		spellRankText, spellRankNumber = strsplit(" ", spellRank)
		if tonumber(spellRankNumber) ~= nil then
			if string.find(spellName, BS["Conjure Water"]) then
				Cryolysis2.Private.CanConjureWater = true
				if tonumber(spellRankNumber) > Cryolysis2.Private.HighestWaterId then
					Cryolysis2.Private.HighestWaterId = tonumber(spellRankNumber)
				end
			end
			if string.find(spellName, BS["Conjure Food"]) then
				Cryolysis2.Private.CanConjureFood = true
				if tonumber(spellRankNumber) > Cryolysis2.Private.HighestFoodId then
					Cryolysis2.Private.HighestFoodId = tonumber(spellRankNumber)
				end
			end
		end
		if string.find(spellName, BS["Ritual of Refreshment"]) then
			Cryolysis2.Private.HasRitual = true
		end
		if spellName == BS["Conjure Mana Agate"] then
			Cryolysis2.Private.CanConjureGem[1] = true
			if 1 > Cryolysis2.Private.HighestManaGemId then
				Cryolysis2.Private.HighestManaGemId = 1
			end
		end
		if spellName == BS["Conjure Mana Jade"] then
			Cryolysis2.Private.CanConjureGem[2] = true
			if 2 > Cryolysis2.Private.HighestManaGemId then
				Cryolysis2.Private.HighestManaGemId = 2
			end
		end
		if spellName == BS["Conjure Mana Citrine"] then
			Cryolysis2.Private.CanConjureGem[3] = true
			if 3 > Cryolysis2.Private.HighestManaGemId then
				Cryolysis2.Private.HighestManaGemId = 3
			end
		end
		if spellName == BS["Conjure Mana Ruby"] then
			Cryolysis2.Private.CanConjureGem[4] = true
			if 4 > Cryolysis2.Private.HighestManaGemId then
				Cryolysis2.Private.HighestManaGemId = 4
			end
		end
		if spellName == BS["Conjure Mana Emerald"] then
			Cryolysis2.Private.CanConjureGem[5] = true
			if 5 > Cryolysis2.Private.HighestManaGemId then
				Cryolysis2.Private.HighestManaGemId = 5
			end
		end
		i = i + 1
	end
end

------------------------------------------------------------------------------------------------------
-- Function to detect how much water we got in the bags
------------------------------------------------------------------------------------------------------
function Cryolysis2:CountWater()

	for i = #(Cryolysis2.Private.WaterName), 1, -1 do
		if GetItemCount(Cryolysis2.Private.WaterName[i]) > 0 then
			return GetItemCount(Cryolysis2.Private.WaterName[i])
		end
	end
	return 0
	
end

------------------------------------------------------------------------------------------------------
-- Function to detect what the highest water we got in the bags
------------------------------------------------------------------------------------------------------
function Cryolysis2:WaterOnHand()

	Cryolysis2.Private.HighestWaterOnHand = 0

	for i= 1, #(Cryolysis2.Private.WaterName), 1 do
		if GetItemCount(Cryolysis2.Private.WaterName[i]) > 0 then
			Cryolysis2.Private.HighestWaterOnHand = i
		end
	end

end

------------------------------------------------------------------------------------------------------
-- Function to detect how much food we got in the bags
------------------------------------------------------------------------------------------------------
function Cryolysis2:CountFood()

	for i = #(Cryolysis2.Private.FoodName), 1, -1 do
		if GetItemCount(Cryolysis2.Private.FoodName[i]) > 0 then
			return GetItemCount(Cryolysis2.Private.FoodName[i])
		end
	end
	return 0
	
end

------------------------------------------------------------------------------------------------------
-- Function to detect what the highest food we got in the bags
------------------------------------------------------------------------------------------------------
function Cryolysis2:FoodOnHand()
	
	Cryolysis2.Private.HighestFoodOnHand = 0
	
	for i= 1, #(Cryolysis2.Private.FoodName), 1 do
		if GetItemCount(Cryolysis2.Private.FoodName[i]) > 0 then
			Cryolysis2.Private.HighestFoodOnHand = i
		end
	end
	
end

function Cryolysis2:FindAllMounts()
    
    local duplicate = false
    
	for icon = 1, #(Cryolysis2.Mounts), 1 do
		for i = 1, #(Cryolysis2.Mounts[icon]), 1 do
			if GetItemCount(Cryolysis2.Mounts[icon][i]) > 0 then
                Cryolysis2.Private.HasMount = true
                if Cryolysis2.db.profile.LeftMount == "" then
                    Cryolysis2.db.profile.LeftMount = Cryolysis2.Mounts[icon][i]
                end
                if Cryolysis2.db.profile.RightMount == "" then
                    Cryolysis2.db.profile.RightMount = Cryolysis2.Mounts[icon][i]
                end
				for j = 1, #(Cryolysis2.Private.Mounts), 1 do
                    if Cryolysis2.Private.Mounts[j] == Cryolysis2.Mounts[icon][i] then
                        duplicate = true
                    end
                end
                if duplicate == false then
                    table.insert(Cryolysis2.Private.Mounts, Cryolysis2.Mounts[icon][i])
                    Cryolysis2:Print("Mount Found: "..Cryolysis2.Mounts[icon][i])
                end
                duplicate = false
			end
		end
	end
    
    Cryolysis2.options.args.graphicalsettings.args.mountbutton.args.leftmount.validate = Cryolysis2.Private.Mounts
    Cryolysis2.options.args.graphicalsettings.args.mountbutton.args.rightmount.validate = Cryolysis2.Private.Mounts

end

function Cryolysis2:FindMountIcon(mountName)

	for icon = 1, #(Cryolysis2.Mounts), 1 do
		for i = 1, #(Cryolysis2.Mounts[icon]), 1 do
			if mountName == Cryolysis2.Mounts[icon][i] then
				return icon
			end
		end
	end

end

------------------------------------------------------------------------------------------------------
-- Function to detect whether or not the player has a ground mount
------------------------------------------------------------------------------------------------------
function Cryolysis2:FindGroundMount()

	for icon = 1, #(Cryolysis2.GroundMounts), 1 do
		for i = 1, #(Cryolysis2.GroundMounts[icon]), 1 do
			if GetItemCount(Cryolysis2.GroundMounts[icon][i]) > 0 then
				Cryolysis2.Private.GroundMountString = Cryolysis2.GroundMounts[icon][i]
				Cryolysis2.Private.GroundMountIcon = icon
				Cryolysis2.Private.HasGroundMount = true
				return;
			end
		end
	end
    
end

------------------------------------------------------------------------------------------------------
-- Function to detect whether or not the player has a flying mount
------------------------------------------------------------------------------------------------------
function Cryolysis2:FindFlyingMount()

	for icon = 1, #(Cryolysis2.FlyingMounts), 1 do
		for i = 1, #(Cryolysis2.FlyingMounts[icon]), 1 do
			if GetItemCount(Cryolysis2.FlyingMounts[icon][i]) > 0 then
				Cryolysis2.Private.FlyingMountString = Cryolysis2.FlyingMounts[icon][i]
				Cryolysis2.Private.FlyingMountIcon = icon
				Cryolysis2.Private.HasFlyingMount = true
				return;
			end
		end
	end
   
end

------------------------------------------------------------------------------------------------------
-- Function to auto-trade food to target
------------------------------------------------------------------------------------------------------
function Cryolysis2:TradeFood()
	
	if (UnitExists("target") and UnitIsPlayer("target") and UnitIsFriend("player", "target") and UnitName("target") ~= UnitName("player")) then
		
		Cryolysis2:FoodOnHand()
		
		if Cryolysis2.Private.HighestFoodOnHand > 0 then
			local itemString, itemName
			for bag = 0, 4, 1 do
				for slot = 1, GetContainerNumSlots(bag), 1 do
					if (GetContainerItemLink(bag,slot)) then
						itemString = ""..GetContainerItemLink(bag,slot)
						itemName = GetItemInfo(itemString)
						_, _, locked = GetContainerItemInfo(bag, slot)
						if itemName == Cryolysis2.Private.FoodName[Cryolysis2.Private.HighestFoodOnHand] and not locked then
							PickupContainerItem(bag, slot)
							if CursorHasItem() then
								if Cryolysis2.Private.TradeWindowOpen then
									DropItemOnUnit("npc");
									return
								else
									DropItemOnUnit("target");
									return
								end
							end
						end
					end
				end
			end
		else
			Cryolysis2:Print(L["No food found for trading, conjure some then try again!"])
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Function to auto-trade water to target
------------------------------------------------------------------------------------------------------
function Cryolysis2:TradeWater()
	if (UnitExists("target") and UnitIsPlayer("target") and UnitIsFriend("player", "target") and UnitName("target") ~= UnitName("player")) then
		
		Cryolysis2:WaterOnHand()
		
		if Cryolysis2.Private.HighestWaterOnHand > 0 then
			local itemString, itemName
			for bag = 0, 4, 1 do
				for slot = 1, GetContainerNumSlots(bag), 1 do
					if (GetContainerItemLink(bag,slot)) then
						itemString = ""..GetContainerItemLink(bag,slot)
						itemName = GetItemInfo(itemString)
						_, _, locked = GetContainerItemInfo(bag, slot)
						if itemName == Cryolysis2.Private.WaterName[Cryolysis2.Private.HighestWaterOnHand] and not locked then
							PickupContainerItem(bag, slot)
							if CursorHasItem() then
								if Cryolysis2.Private.TradeWindowOpen then
									DropItemOnUnit("npc");
									return
								else
									DropItemOnUnit("target");
									return
								end
							end
						end
					end
				end
			end
		else
			Cryolysis2:Print(L["No water found for trading, conjure some then try again!"])
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Function for restocking reagents when visiting vendor
------------------------------------------------------------------------------------------------------
function Cryolysis2:RestockReagents()

	local PowderIndex, TeleportIndex, PortalsIndex
	for i = 1, GetMerchantNumItems(), 1 do
		local name, texture, price, quantity, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(i)
		if name == L["Arcane Powder"] then
			PowderIndex = i
		end
		if name == L["Rune of Teleportation"] then
			TeleportIndex = i
		end
		if name == L["Rune of Portals"] then
			PortalsIndex = i
		end
	end

	if Cryolysis2.Private.HasArcaneBrilliance then
		local tempArcanePowder = GetItemCount(L["Arcane Powder"])
		if tempArcanePowder < Cryolysis2.db.profile.RestockPowder then
			local calcAP = Cryolysis2.db.profile.RestockPowder - tempArcanePowder
			while calcAP > 20 do
				BuyMerchantItem(PowderIndex, 20)
				calcAP = calcAP - 20
			end
			BuyMerchantItem(PowderIndex, calcAP)
		end
	end
	if Cryolysis2.Private.HasATeleport then
		local tempTeleport = GetItemCount(L["Rune of Teleportation"])
		if tempTeleport < Cryolysis2.db.profile.RestockTeleport then
			local calcTele = Cryolysis2.db.profile.RestockTeleport - tempTeleport
			while calcTele > 10 do
				BuyMerchantItem(TeleportIndex, 10)
				calcTele = calcTele - 10
			end
			BuyMerchantItem(TeleportIndex, calcTele)
		end
	end
	if Cryolysis2.Private.HasAPort then
		local tempPortals = GetItemCount(L["Rune of Portals"])
		if tempPortals < Cryolysis2.db.profile.RestockPortals then
			local calcPort = Cryolysis2.db.profile.RestockPortals - tempPortals
			while calcPort > 10 do
				BuyMerchantItem(PortalsIndex, 10)
				calcPort = calcPort - 10
			end
			BuyMerchantItem(PortalsIndex, calcPort)
		end
	end

end

------------------------------------------------------------------------------------------------------
-- Here be dragons.
------------------------------------------------------------------------------------------------------
function Cryolysis2:Polymorph()



end